/*
    Copyright (C) 2001-2003 KSVG Team
    This file is part of the KDE project

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    aint with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#ifndef KSVGHelper_H
#define KSVGHelper_H

#include <qcolor.h>
#include "SVGMatrixImpl.h"

namespace KSVG
{

class KSVGHelper
{
public:
	static void matrixToAffine(const SVGMatrixImpl *matrix, double *affine)
	{
		return KSVGHelper::matrixToAffine(const_cast<SVGMatrixImpl *>(matrix), affine);
	}

	static void matrixToAffine(SVGMatrixImpl *matrix, double *affine)
	{
		affine[0] = matrix->a();
		affine[1] = matrix->b();
		affine[2] = matrix->c();
		affine[3] = matrix->d();
		affine[4] = matrix->e();
		affine[5] = matrix->f();
	}

	static QString toColorString(QColor color)
	{
		int r = color.red();
		int g = color.green();
		int b = color.blue();

		return "rgb(" + QString::number(r) + "," + QString::number(g) + "," + QString::number(b) + ")";
	}

	static unsigned int toArtColor(const QColor &color)
	{
		return (qRed(color.rgb()) << 24) | (qGreen(color.rgb()) << 16) | ( qBlue(color.rgb()) << 8) | (qAlpha(color.rgb()));
	}

	static unsigned int toArtColor(const QColor &color, short opacity)
	{
		return (qRed(color.rgb()) << 24) | (qGreen(color.rgb()) << 16) | ( qBlue(color.rgb()) << 8) | (opacity);
	}
};

class KSVGPoint
{
public:
	KSVGPoint() { m_x = 0.0; m_y = 0.0; }
	KSVGPoint(double x, double y) { m_x = x; m_y = y; }
	KSVGPoint &operator=(const KSVGPoint &other) { m_x = other.x(); m_y = other.y(); return *this; }

	friend inline KSVGPoint operator+(const KSVGPoint &, const KSVGPoint &);
	friend inline KSVGPoint operator-(const KSVGPoint &, const KSVGPoint &);
	friend inline KSVGPoint operator*(const double &, const KSVGPoint &);
	friend inline KSVGPoint operator*(const KSVGPoint &, const double &);

	double x() const { return m_x; }
	double y() const { return m_y; }

	void setX(double x) { m_x = x; }
	void setY(double y) { m_y = y; }

private:
        double m_x;
        double m_y;
};

inline KSVGPoint operator+(const KSVGPoint &p1, const KSVGPoint &p2 )
{ return KSVGPoint(p1.m_x + p2.m_x, p1.m_y + p2.m_y); }

inline KSVGPoint operator-(const KSVGPoint &p1, const KSVGPoint &p2 )
{ return KSVGPoint(p1.m_x - p2.m_x, p1.m_y - p2.m_y); }

inline KSVGPoint operator*(const double &c, const KSVGPoint &p)
{ return KSVGPoint(p.m_x * c, p.m_y * c); }

inline KSVGPoint operator*(const KSVGPoint &p, const double &c)
{ return KSVGPoint(p.m_x * c, p.m_y * c); }

};

#endif

// vim:ts=4:noet
