/*
    Copyright (C) 2003 Nikolas Zimmermann <wildfox@kde.org>
    This file is part of the KDE project

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    aint with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#ifndef T2P_TOOLS_H
#define T2P_TOOLS_H

#include <list>
#include <string>
#include <sstream>

#define T2PMAX(a, b) ((b) < (a) ? (a) : (b))
#define T2PMIN(a, b) ((a) < (b) ? (a) : (b))

namespace T2P
{
	typedef struct
	{
		int x0, y0, x1, y1;
	} BBoxData;

	class Tools
	{
	public:
		// Finds the smallest rectangle that includes src1 and src2.
		static void bboxUnion(BBoxData *dest, const BBoxData *src1, const BBoxData *src2)
		{
			if(src1->x1 <= src1->x0 || src1->y1 <= src1->y0) // Is src1 empty?
			{ // Copy src2 to dest
				dest->x0 = src2->x0;
				dest->y0 = src2->y0;
				dest->x1 = src2->x1;
				dest->y1 = src2->y1;
			}
			else if(src2->x1 <= src2->x0 || src2->y1 <= src2->y0) // Is src2 empty?
			{ // Copy src1 to dest
				dest->x0 = src1->x0;
				dest->y0 = src1->y0;
				dest->x1 = src1->x1;
				dest->y1 = src1->y1;
			}
			else
			{
				dest->x0 = T2PMIN(src1->x0, src2->x0);
				dest->y0 = T2PMIN(src1->y0, src2->y0);
				dest->x1 = T2PMAX(src1->x1, src2->x1);
				dest->y1 = T2PMAX(src1->y1, src2->y1);
			}
		}
		
		static std::string joinList(char seperator, std::list<std::string> &list)
		{
			std::string result;
			
			if(list.empty())
				return result;
			 
			bool first = true;
			for(std::list<std::string>::const_iterator it = list.begin(); it != list.end(); ++it)
			{
				std::string string = *it;

				if(!string.empty())
				{
					if(!first)
						result += seperator + string;
					else
					{
						result += string;
						first = false;
					}
				}
			}

			return result;
		}

		template<typename T>
		static std::string a2str(T arg)
		{
			std::ostringstream buffer;
			buffer << arg;
			return buffer.str();
		}
	};
};

#endif

// vim:ts=4:noet
