/*
    Copyright (C) 2001-2003 KSVG Team
    This file is part of the KDE project

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    aint with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#include "LibartCanvas.h"
#include "SVGMatrixImpl.h"
#include "SVGRectImpl.h"
#include "SVGPaintImpl.h"
#include "SVGShapeImpl.h"
#include "SVGDocumentImpl.h"
#include "SVGSVGElementImpl.h"
#include "SVGStringListImpl.h"
#include "SVGPatternElementImpl.h"
#include "SVGGradientElementImpl.h"
#include "SVGLinearGradientElementImpl.h"
#include "SVGRadialGradientElementImpl.h"
#include "SVGPatternElementImpl.h"
#include "SVGClipPathElementImpl.h"
#include "SVGTextPositioningElementImpl.h"
#include "SVGAnimatedLengthImpl.h"
#include "SVGAnimatedLengthListImpl.h"
#include "SVGAnimatedEnumerationImpl.h"

#include <kdebug.h>
#include <kglobal.h>
#include <kgenericfactory.h>

#include "SVGPaint.h"

#include <qdatetime.h>
#include <qstring.h>
#include <qimage.h>

#include "KSVGHelper.h"
#include "KSVGTextChunk.h"
#include "LibartCanvasItems.h"

#include <libart_lgpl/art_rgb.h>
#include <libart_lgpl/art_affine.h>
#include <libart_lgpl/art_alphagamma.h>
#include <libart_lgpl/art_rgb_svp.h>
#include <libart_lgpl/art_svp.h>
#include <libart_lgpl/art_svp_ops.h>
#include <libart_lgpl/art_svp_intersect.h>
#include <libart_lgpl/art_rect_svp.h>
#include <libart_lgpl/art_svp_vpath.h>

#include <libs/art_support/art_misc.h>
#include <libs/art_support/art_rgba_svp.h>

#include <Font.h>
#include "BezierPathLibart.h"
#include "GlyphTracerLibart.h"

#include <fontconfig/fontconfig.h>

ArtSVP *art_svp_from_rect(int x0, int y0, int x1, int y1)
{
	ArtVpath vpath[] =
	{
		{ ART_MOVETO, x0, y0 },
		{ ART_LINETO, x0, y1 },
		{ ART_LINETO, x1, y1 },
		{ ART_LINETO, x1, y0 },
		{ ART_LINETO, x0, y0 },
		{ ART_END, 0, 0}
	};

	return art_svp_from_vpath(vpath);
} 

ArtSVP *art_svp_from_irect(ArtIRect *bbox)
{
	return art_svp_from_rect(bbox->x0, bbox->y0, bbox->x1, bbox->y1);
} 

using namespace KSVG;

LibartCanvas::LibartCanvas(int width, int height) : KSVGCanvas(width, height)
{
	m_fontContext = new T2P::Converter(new T2P::GlyphTracerLibart());
}

// drawing primitives
CanvasItem *LibartCanvas::createRectangle(SVGRectElementImpl *rect)
{
	CanvasItem *item = new LibartRectangle(this, rect);
	insert(item);
	return item;
}

CanvasItem *LibartCanvas::createEllipse(SVGEllipseElementImpl *ellipse)
{
	CanvasItem *item = new LibartEllipse(this, ellipse);
	insert(item);
	return item;
}

CanvasItem *LibartCanvas::createCircle(SVGCircleElementImpl *circle)
{
	CanvasItem *item = new LibartCircle(this, circle);
	insert(item);
	return item;
}

CanvasItem *LibartCanvas::createLine(SVGLineElementImpl *line)
{
	CanvasItem *item = new LibartLine(this, line);
	insert(item);
	return item;
}

CanvasItem *LibartCanvas::createPolyline(SVGPolylineElementImpl *poly, int points)
{
	CanvasItem *item = new LibartPolyline(this, poly, points);
	insert(item);
	return item;
}

CanvasItem *LibartCanvas::createPolygon(SVGPolygonElementImpl *poly)
{
	CanvasItem *item = new LibartPolygon(this, poly);
	insert(item);
	return item;
}

CanvasItem *LibartCanvas::createPath(SVGPathElementImpl *path)
{
	CanvasItem *item = new LibartPath(this, path);
	insert(item);
	return item;
}

CanvasItem *LibartCanvas::createClipPath(SVGClipPathElementImpl *clippath)
{
	CanvasClipPath *result = new LibartClipPath(this, clippath);
	QString index = clippath->id().string();
	m_clipPaths.insert(index, result);
	return result;
}

CanvasItem *LibartCanvas::createImage(SVGImageElementImpl *image)
{
	CanvasItem *result = new LibartImage(this, image);
	result->setZIndex(m_chunksByItem.size());
	kdDebug(26005) << "Creating image with z : " << m_chunksByItem.size() << endl;
	m_chunksByItem[result] = QPtrList<CanvasChunk>();
	addToChunks(result);
	m_items.append(result);
	return result;
}

CanvasItem *LibartCanvas::createCanvasMarker(SVGMarkerElementImpl *marker)
{
	CanvasItem *item = new LibartMarker(this, marker);
	insert(item);
	return item;
}

CanvasItem *LibartCanvas::createText(SVGTextElementImpl *text)
{
	CanvasItem *item = new LibartText(this, text);
	insert(item);
	return item;
}

CanvasItem *LibartCanvas::createPaintServer(const QString &id, SVGElementImpl *pserver)
{
	LibartPaintServer *result;
	if(dynamic_cast<SVGLinearGradientElementImpl *>(pserver))
		result = new LibartLinearGradient(dynamic_cast<SVGLinearGradientElementImpl *>(pserver));
	else if(dynamic_cast<SVGRadialGradientElementImpl *>(pserver))
		result = new LibartRadialGradient(dynamic_cast<SVGRadialGradientElementImpl *>(pserver));
	else if(dynamic_cast<SVGPatternElementImpl *>(pserver))
		result = new LibartPattern(dynamic_cast<SVGPatternElementImpl *>(pserver));
	m_paintServers.insert(id, result);
	return 0;
}

void LibartCanvas::drawImage(QImage image, SVGStylableImpl *style, const SVGMatrixImpl *matrix)
{
	if(image.depth() != 32)
		image = image.convertDepth(32);

	ArtSVP *imageBorder = clippingRect(QRect(0, 0, image.width(), image.height()), matrix);
	ArtSVP *clipSvp = clipSingleSVP(imageBorder, style->getClipPath(), dynamic_cast<SVGShapeImpl *>(style));

	ArtDRect bbox;
	art_drect_svp(&bbox, clipSvp);

	// clamp to viewport
	int x0 = int(bbox.x0);
	int y0 = int(bbox.y0);

	// Use inclusive coords for x1/y1 for clipToBuffer
	int x1 = int(ceil(bbox.x1)) - 1;
	int y1 = int(ceil(bbox.y1)) - 1;

	kdDebug(26005) << "viewbox : " << x0 << ", " << y0 << ", " << x1 << ", " << y1 << endl;

	if(x0 < m_width && y0 < m_height && x1 >= 0 && y1 >= 0)
	{
		clipToBuffer(x0, y0, x1, y1);

		double affine[6];
		KSVGHelper::matrixToAffine(matrix, affine);

		ksvg_art_rgb_affine_clip(clipSvp, m_buffer + x0 * 3 + y0 * 3 * m_width, x0, y0, x1 + 1, y1 + 1, m_width * 3, image.bits(), image.width(), image.height(), image.width() * 4, affine, ART_FILTER_NEAREST, 0L, int(style->getOpacity() * 255));
	}

	art_svp_free(imageBorder);

	if(clipSvp != imageBorder)
		art_svp_free(clipSvp);
}

ArtSVP *LibartCanvas::clippingRect(const QRect &rect, const SVGMatrixImpl *ctm)
{
	ArtVpath *vec = allocVPath(6);

	vec[0].code = ART_MOVETO;
	vec[0].x = rect.x();
	vec[0].y = rect.y();

	vec[1].code = ART_LINETO;
	vec[1].x = rect.x();
	vec[1].y = rect.y() + rect.height();

	vec[2].code = ART_LINETO;
	vec[2].x = rect.x() + rect.width();
	vec[2].y = rect.y() + rect.height();

	vec[3].code = ART_LINETO;
	vec[3].x = rect.x() + rect.width();
	vec[3].y = rect.y();

	vec[4].code = ART_LINETO;
	vec[4].x = rect.x();
	vec[4].y = rect.y();

	vec[5].code = ART_END;

	double affine[6];
	KSVGHelper::matrixToAffine(ctm, affine);

	ArtVpath *temp = art_vpath_affine_transform(vec, affine);
	art_free(vec);

	ArtSVP *result = art_svp_from_vpath(temp);
	art_free(temp);
	return result;
}

ArtSVP *LibartCanvas::clipSingleSVP(ArtSVP *svp, const QString &clipRef, SVGShapeImpl *clipShape)
{
	SVGElementImpl *element = dynamic_cast<SVGElementImpl *>(clipShape);

	ArtSVP *temp = svp;
	CanvasClipPath *clip = 0;
	if(!clipRef.isEmpty())
	{
		clip = m_clipPaths[clipRef];
		if(clip)
		{
			LibartClipPath *lclip = dynamic_cast<LibartClipPath *>(clip);
			reinterpret_cast<SVGClipPathElementImpl *>(clip->element())->setBBoxTarget(clipShape);

			lclip->init();
			temp = art_svp_intersect(lclip->clipSVP(), svp);
		}
	}

	if(element->ownerSVGElement())
	{
		ArtSVP *svgClip = clippingRect(element->ownerSVGElement()->clip(), element->ownerSVGElement()->screenCTM());
		ArtSVP *result = art_svp_intersect(svgClip, temp);
		if(temp != svp)
			art_svp_free(temp);

		art_svp_free(svgClip);
		return result;
	}

	return temp;
}

void LibartCanvas::drawSVP(ArtSVP *svp, art_u32 color, const QString &clipRef, SVGShapeImpl *clipShape)
{
	if(!svp)
		return;

	ArtDRect bbox;
	ArtSVP *clip = clipSingleSVP(svp, clipRef, clipShape);

	// Clipping
	art_drect_svp(&bbox, clip);

	// clamp to viewport
	int x0 = int(bbox.x0);
	int y0 = int(bbox.y0);

	// Use inclusive coords for x1/y1 for clipToBuffer
	int x1 = int(ceil(bbox.x1)) - 1;
	int y1 = int(ceil(bbox.y1)) - 1;

	//kdDebug(26005) << "viewbox : " << x0 << ", " << y0 << ", " << x1 << ", " << y1 << endl;
	if(x0 <= m_width || y0 <= m_height || x1 > -1 || y1 > -1)
	{
		clipToBuffer(x0, y0, x1, y1);

		if(m_nrChannels == 3)
			art_rgb_svp_alpha(clip, x0, y0, x1 + 1, y1 + 1, color, m_buffer + x0 * 3 + y0 * 3 * m_width, m_width * 3, 0);
		else
			art_rgba_svp_alpha(clip, x0, y0, x1 + 1, y1 + 1, color >> 8, 255, m_buffer + x0 * 4 + y0 * 4 * m_width, m_width * 4, 0);
	}
	if(clip != svp)
		art_svp_free(clip);
}

ArtSVP *LibartCanvas::copy_svp(const ArtSVP *svp)
{
	// No API to copy an SVP so do so without accessing the data structure directly.
	ArtVpath *vec = allocVPath(1);

	vec[0].code = ART_END;

	ArtSVP *empty = art_svp_from_vpath(vec);
	art_free(vec);

	ArtSVP *result = art_svp_union(empty, svp);
	art_svp_free(empty);

	return result;
}


// vim:ts=4:noet
